@@ -21,17 +21,7 @@
  */
 
 #include <config.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <math.h>
-#include <sys/errno.h>
-#include <sys/ioctl.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <unistd.h>
-#include <string.h>
-#include <signal.h>
-#include <ctype.h>
+
 #ifdef ALSA
 #include <sys/asoundlib.h>
 #else
@@ -55,86 +45,6 @@ mixerprefs prefs={TRUE, -1, TRUE, -1, TRUE, -1};
 static GtkWidget *configwin = NULL;
 static GConfClient *client = NULL;
 
-static void
-restore_toggled (GtkToggleButton *tb,
-		 gpointer data)
-{
-	gconf_client_set_bool (client, "/apps/gnome-volume-control/init-on-start",
-			       gtk_toggle_button_get_active (tb), NULL);
-}
-
-static void
-show_icons_toggled (GtkToggleButton *tb,
-		    gpointer data)
-{
-	gconf_client_set_bool (client, "/apps/gnome-volume-control/show-icons",
-			       gtk_toggle_button_get_active (tb), NULL);
-}
-
-static void
-show_labels_toggled (GtkToggleButton *tb,
-		     gpointer data)
-{
-	gconf_client_set_bool (client, "/apps/gnome-volume-control/show-labels",
-			       gtk_toggle_button_get_active (tb), NULL);
-}
-			       
-static GtkWidget *general_page(void)
-{
-	GtkWidget *start_frame, *gui_frame;
-	GtkWidget *ubervbox;
-	GtkWidget *vbox, *init_start, *menu_hide, *temp;
-
-	ubervbox = gtk_vbox_new (FALSE, 0);
-
-	start_frame = gtk_frame_new(_("On startup"));
-	gtk_container_border_width(GTK_CONTAINER(start_frame), GNOME_PAD_SMALL);
-    
-	vbox = gtk_vbox_new(TRUE, 0);
-
-	/* Set on start */
-	init_start = gtk_check_button_new_with_label(_("Restore saved mixer levels on startup"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (init_start),
-				      prefs.set_mixer_on_start);
-	gtk_signal_connect(GTK_OBJECT(init_start), "toggled",
-			   GTK_SIGNAL_FUNC(restore_toggled), NULL);
-
-	gtk_box_pack_start_defaults(GTK_BOX(vbox), init_start);
-	gtk_widget_show_all(vbox);
-
-
-	gtk_container_add (GTK_CONTAINER(start_frame), vbox);
-	gtk_box_pack_start (GTK_BOX (ubervbox), start_frame, FALSE, FALSE, 0);
-
-	gui_frame = gtk_frame_new(_("GUI"));
-	gtk_container_border_width(GTK_CONTAINER(gui_frame), GNOME_PAD_SMALL);
-
-	vbox = gtk_vbox_new(TRUE, 0);
-	temp = gtk_check_button_new_with_label(_("Show mixer icons"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
-				     prefs.show_icons);
-	gtk_signal_connect(GTK_OBJECT(temp), "toggled",
-			   GTK_SIGNAL_FUNC(show_icons_toggled), NULL);
-	gtk_box_pack_start_defaults(GTK_BOX(vbox), temp);
-	
-	
-	temp = gtk_check_button_new_with_label(_("Show mixer labels"));
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(temp),
-				     prefs.show_labels);
-	gtk_signal_connect(GTK_OBJECT(temp), "toggled",
-			   GTK_SIGNAL_FUNC(show_labels_toggled), NULL);
-	gtk_box_pack_start_defaults(GTK_BOX(vbox), temp);
-	
-	
-	gtk_container_add(GTK_CONTAINER(gui_frame), vbox);
-	
-	gtk_box_pack_start (GTK_BOX (ubervbox), gui_frame, FALSE, FALSE, 0);
-	
-	gtk_widget_show_all(ubervbox);
-
-	return ubervbox;
-}
-
 /******************** Label Name Page ********************/
 
 static gboolean
@@ -253,28 +163,10 @@ edited (GtkCellRendererText *cell,
 	gtk_tree_path_free (path);
 }
 
-/*
- * Whip up a page showing all the labels for a single sound device.
- * This page is presented as a vertically scrolling pane containing a
- * table of channels and their labels, and button to reset all the
- * labels to their default values.  Each row in the table is a single
- * sound channel giving its "name" (gtk label) and its current value
- * (gtk entry).  All of the gtk entries are strung onto a list for
- * easy access later in the callback routines.  A clist would have
- * been convenient for this task except that it doesn't support
- * editable items.
- *
- * @param device A pointer to the device_info data structure for this
- * sound device.
- *
- * @param parent A pointer to the parent notebook that contains a
- * single page for each sound device.
- */
-static void
-label_one_device_config (gpointer device,
-			 gpointer parent)
+static GtkWidget *
+label_one_device_config_widget (gpointer device)
 {
-	GtkWidget *ubervbox, *label, *treeview, *bbox, *button, *scrlwin;
+	GtkWidget *ubervbox, *treeview, *bbox, *button, *scrlwin;
 	GtkWidget *topvbox;
 	GtkTreeModel *model;
 	GtkCellRenderer *cell;
@@ -282,17 +174,16 @@ label_one_device_config (gpointer device,
 	
 	device_info *info = (device_info *) device;
 
-	/* The page: A scrolled viewport (and a label) */
-	label = gtk_label_new(info->info.name);
+	/* The page: A scrolled viewport */
 	scrlwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrlwin),
 				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrlwin),
+					    GTK_SHADOW_IN);  
 
 	topvbox = gtk_vbox_new (FALSE, 5);
 	gtk_box_pack_start (GTK_BOX (topvbox), scrlwin, TRUE, TRUE, 0);
 	
-	gtk_notebook_append_page(GTK_NOTEBOOK(parent), topvbox, label);
-
 	model = make_label_model (info->channels);
 	treeview = gtk_tree_view_new_with_model (model);
 
@@ -326,6 +217,40 @@ label_one_device_config (gpointer device,
 	gtk_box_pack_start (GTK_BOX(topvbox), button, FALSE, FALSE, 0);
 	g_signal_connect (G_OBJECT(button), "clicked",
 			  G_CALLBACK (labels_reset_page_cb), model);
+
+	return topvbox;
+}
+
+/*
+ * Whip up a page showing all the labels for a single sound device.
+ * This page is presented as a vertically scrolling pane containing a
+ * table of channels and their labels, and button to reset all the
+ * labels to their default values.  Each row in the table is a single
+ * sound channel giving its "name" (gtk label) and its current value
+ * (gtk entry).  All of the gtk entries are strung onto a list for
+ * easy access later in the callback routines.  A clist would have
+ * been convenient for this task except that it doesn't support
+ * editable items.
+ *
+ * @param device A pointer to the device_info data structure for this
+ * sound device.
+ *
+ * @param parent A pointer to the parent notebook that contains a
+ * single page for each sound device.
+ */
+static void
+label_one_device_config (gpointer device,
+			 gpointer parent)
+{
+	GtkWidget *topvbox;
+	GtkWidget *label;
+
+	device_info *info = (device_info *) device;
+	
+	topvbox = label_one_device_config_widget (device);
+	label = gtk_label_new(info->info.name);
+
+	gtk_notebook_append_page(GTK_NOTEBOOK(parent), topvbox, label);
 }
 
 /*
@@ -341,18 +266,24 @@ label_one_device_config (gpointer device,
  */
 static GtkWidget *labels_create_page(void)
 {
-	GtkWidget *notebook;
-
-	notebook = gtk_notebook_new();
-	gtk_container_set_border_width(GTK_CONTAINER(notebook), 5);
+	GtkWidget *page;
 
 	if (devices->next == NULL) {
 		/* devices->next will only be NULL if there's only one device. */
-		gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
+		GtkWidget *widget;
+
+		widget = label_one_device_config_widget (devices->data);
+		page = gtk_hbox_new (FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (page), widget, TRUE, TRUE, 0);
+		gtk_container_set_border_width (GTK_CONTAINER (page), 5);
+	} else {
+		page = gtk_notebook_new();
+		gtk_container_set_border_width(GTK_CONTAINER(page), 5);
+
+		g_list_foreach(devices, label_one_device_config, page);
 	}
-	
-	g_list_foreach(devices, label_one_device_config, notebook);
-	return(notebook);
+
+	return(page);
 }
 
 static void
@@ -373,7 +304,7 @@ prefs_destroy_cb (GtkWidget *widget,
 void
 prefs_make_window (GtkWidget *toplevel)
 {
-	GtkWidget *label, *page, *notebook;
+	GtkWidget *page;
 
 	if (configwin != NULL) {
 		gdk_window_show (configwin->window);
@@ -384,6 +315,7 @@ prefs_make_window (GtkWidget *toplevel)
 							 GTK_DIALOG_DESTROY_WITH_PARENT,
 							 GTK_STOCK_CLOSE,
 							 GTK_RESPONSE_CLOSE, NULL);
+		gtk_window_set_default_size (GTK_WINDOW (configwin), 380, 256);
 		g_signal_connect (G_OBJECT (configwin), "response",
 				  G_CALLBACK (prefs_response_cb), NULL);
 		g_signal_connect (G_OBJECT (configwin), "destroy",
@@ -391,50 +323,15 @@ prefs_make_window (GtkWidget *toplevel)
 		g_signal_connect (G_OBJECT (configwin), "destroy",
 				  G_CALLBACK (gtk_widget_destroyed), &configwin);
 
-		notebook = gtk_notebook_new ();
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (configwin)->vbox), notebook, TRUE, TRUE, 0);
-
-		label = gtk_label_new (_("Preferences"));
-		page = general_page ();
-		gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
-
-		label = gtk_label_new (_("Labels"));
 		page = labels_create_page ();
-		gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (configwin)->vbox),
+				    page, TRUE, TRUE, 0);
 		
 		gtk_widget_show_all (configwin);
 	}
 }
 
-static void
-init_on_start_changed (GConfClient *_client,
-		       guint cnxn_id,
-		       GConfEntry *entry,
-		       gpointer data)
-{
-	prefs.set_mixer_on_start = gconf_client_get_bool (client, "/apps/gnome-volume-control/init-on-start", NULL);
-}
-
-static void
-show_icons_changed (GConfClient *_client,
-		    guint cnxn_id,
-		    GConfEntry *entry,
-		    gpointer data)
-{
-	prefs.show_icons = gconf_client_get_bool (client, "/apps/gnome-volume-control/show-icons", NULL);
-	gmix_change_icons (prefs.show_icons);
-}
-
-static void
-show_labels_changed (GConfClient *_client,
-		     guint cnxn_id,
-		     GConfEntry *entry,
-		     gpointer data)
-{
-	prefs.show_labels = gconf_client_get_bool (client, "/apps/gnome-volume-control/show-labels", NULL);
-	gmix_change_labels (prefs.show_labels);
-}
-
+/* Not sure if this is needed... */
 void get_gui_config(void)
 {
 	if (client == NULL) {
@@ -443,18 +340,4 @@ void get_gui_config(void)
 
 	gconf_client_add_dir (client, "/apps/gnome-volume-control",
 			      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-	prefs.set_mixer_on_start = gconf_client_get_bool (client, "/apps/gnome-volume-control/init-on-start", NULL);
-	prefs.mixer_id = gconf_client_notify_add (client,
-						  "/apps/gnome-volume-control/init-on-start",
-						  init_on_start_changed, NULL, NULL, NULL);
-
-	prefs.show_icons = gconf_client_get_bool (client, "/apps/gnome-volume-control/show-icons", NULL);
-	prefs.icons_id = gconf_client_notify_add (client,
-						  "/apps/gnome-volume-control/show-icons",
-						  show_icons_changed, NULL, NULL, NULL);
-	
-	prefs.show_labels = gconf_client_get_bool (client, "/apps/gnome-volume-control/show-labels", NULL);
-	prefs.labels_id = gconf_client_notify_add (client,
-						   "/apps/gnome-volume-control/show-labels",
-						   show_labels_changed, NULL, NULL, NULL);
 }
