@@ -296,7 +296,9 @@ cdrom_set_device (GnomeCDRom *cdrom,
 {
 	GnomeCDRomStatus *status;
 	GnomeCDRomPrivate *priv;
-	
+
+	g_return_val_if_fail (cdrom != NULL, FALSE);
+
 	priv = cdrom->priv;
 
 	if (priv->device && !strcmp (priv->device, device))
@@ -304,19 +306,23 @@ cdrom_set_device (GnomeCDRom *cdrom,
 
 	gnome_cdrom_close_dev (cdrom, TRUE);
 	
+	g_free (priv->device);
+	priv->device = g_strdup (device);
 	if (gnome_cdrom_get_status (cdrom, &status, NULL) == TRUE) {
 		if (status->audio == GNOME_CDROM_AUDIO_PLAY) {
 			if (gnome_cdrom_stop (cdrom, error) == FALSE) {
+				g_free (status);
 				return FALSE;
 			}
 		}
+
+		g_free (status);
 	}
 
-	g_free (priv->device);
-	priv->device = g_strdup (device);
 
 	switch (cdrom->lifetime) {
 		case  GNOME_CDROM_DEVICE_STATIC :
+		case  GNOME_CDROM_DEVICE_TRANSIENT :
 			if (!gnome_cdrom_open_dev (cdrom, error))
 				return FALSE;
 			gnome_cdrom_close_dev (cdrom, FALSE);
@@ -332,12 +338,11 @@ cdrom_set_device (GnomeCDRom *cdrom,
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_NOT_OPENED,
 					      "%s does not point to a valid CDRom device. This may be caused by:\n"
-					      "a) CD support is not compiled into Linux\n"
+					      "a) CD support is not present in your machine\n"
 					      "b) You do not have the correct permissions to access the CD drive\n"
 					      "c) %s is not the CD drive.\n",					     
 					      cdrom->priv->device, cdrom->priv->device);
 		}
-		if (cdrom->lifetime == GNOME_CDROM_DEVICE_STATIC)
 			return FALSE;
 	}
 
@@ -366,7 +371,7 @@ cdrom_open_dev (GnomeCDRom *cdrom,
 					GNOME_CDROM_ERROR,
 					GNOME_CDROM_ERROR_NOT_OPENED,
 					_("%s does not appear to point to a valid CD device. This may be because:\n"
-					  "a) CD support is not compiled into Linux\n"
+					  "a) CD support is not present in your machine\n"
 					  "b) You do not have the correct permissions to access the CD drive\n"
 					  "c) %s is not the CD drive.\n"),					     
 					cdrom->priv->device,
@@ -449,6 +454,7 @@ cdrom_init (GnomeCDRom *cdrom)
 	cdrom->priv = g_new0 (GnomeCDRomPrivate, 1);
 	cdrom->playmode = GNOME_CDROM_WHOLE_CD;
 	cdrom->loopmode = GNOME_CDROM_PLAY_ONCE;
+	cdrom->priv->device = g_strdup (default_cd_device);
 }
 
 /* API */
@@ -774,14 +780,18 @@ timeout_update_cd (gpointer data)
 	if (gnome_cdrom_get_status (GNOME_CDROM (cdrom), &status, &error) == FALSE) {
 		g_free (priv->recent_status);
 		priv->recent_status = not_ready_status_new ();
-		if (status->cd == GNOME_CDROM_STATUS_NO_DISC ||
-		    status->cd == GNOME_CDROM_STATUS_NO_CDROM )
-			priv->recent_status->cd = status->cd;
+		if (status != NULL) {
+			if (status->cd == GNOME_CDROM_STATUS_NO_DISC ||
+			    status->cd == GNOME_CDROM_STATUS_NO_CDROM )
+				priv->recent_status->cd = status->cd;
+			g_free (status);
+		}
 		if (priv->update != GNOME_CDROM_UPDATE_NEVER)
 			gnome_cdrom_status_changed (GNOME_CDROM (cdrom), priv->recent_status);
 		gcd_warning ("%s", error);
 		g_error_free (error);
 		g_object_unref (G_OBJECT (cdrom));
+
 		return TRUE;
 	}
 	
@@ -829,7 +839,8 @@ gnome_cdrom_construct (GnomeCDRom      *cdrom,
 	cdrom->lifetime = lifetime;
 	cdrom->priv->update = update;
 
-	if (!gnome_cdrom_set_device (cdrom, device, error)) {
+	if (!gnome_cdrom_set_device (cdrom, device, error) &&
+	    lifetime != GNOME_CDROM_DEVICE_TRANSIENT) {
 		g_object_unref (cdrom);
 		return NULL;
 	}
